/*
Copyright (c) 2019-2022 Digital Energy Cloud Solutions LLC. All Rights Reserved.
Authors:
Petr Krutov, <petr.krutov@digitalenergy.online>
Stanislav Solovev, <spsolovev@digitalenergy.online>
Kasim Baybikov, <kmbaybikov@basistech.ru>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package provider

import (
	"github.com/hashicorp/terraform-plugin-sdk/v2/helper/schema"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/service/cloudapi/account"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/service/cloudapi/bservice"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/service/cloudapi/disks"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/service/cloudapi/flipgroup"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/service/cloudapi/image"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/service/cloudapi/k8s"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/service/cloudapi/kvmvm"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/service/cloudapi/lb"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/service/cloudapi/pfw"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/service/cloudapi/rg"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/service/cloudapi/snapshot"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/service/cloudapi/vins"

	cb_account "repository.basistech.ru/BASIS/terraform-provider-decort/internal/service/cloudbroker/account"
	cb_disks "repository.basistech.ru/BASIS/terraform-provider-decort/internal/service/cloudbroker/disks"
	cb_image "repository.basistech.ru/BASIS/terraform-provider-decort/internal/service/cloudbroker/image"
	cb_k8s "repository.basistech.ru/BASIS/terraform-provider-decort/internal/service/cloudbroker/k8s"
	cb_kvmvm "repository.basistech.ru/BASIS/terraform-provider-decort/internal/service/cloudbroker/kvmvm"
	cb_pcidevice "repository.basistech.ru/BASIS/terraform-provider-decort/internal/service/cloudbroker/pcidevice"
	cb_pfw "repository.basistech.ru/BASIS/terraform-provider-decort/internal/service/cloudbroker/pfw"
	cb_rg "repository.basistech.ru/BASIS/terraform-provider-decort/internal/service/cloudbroker/rg"
	cb_sep "repository.basistech.ru/BASIS/terraform-provider-decort/internal/service/cloudbroker/sep"
	cb_snapshot "repository.basistech.ru/BASIS/terraform-provider-decort/internal/service/cloudbroker/snapshot"
	cb_vins "repository.basistech.ru/BASIS/terraform-provider-decort/internal/service/cloudbroker/vins"
)

func newResourcesMap() map[string]*schema.Resource {
	return map[string]*schema.Resource{
		"decort_resgroup":          rg.ResourceResgroup(),
		"decort_kvmvm":             kvmvm.ResourceCompute(),
		"decort_disk":              disks.ResourceDisk(),
		"decort_disk_snapshot":     disks.ResourceDiskSnapshot(),
		"decort_vins":              vins.ResourceVins(),
		"decort_pfw":               pfw.ResourcePfw(),
		"decort_k8s":               k8s.ResourceK8s(),
		"decort_k8s_wg":            k8s.ResourceK8sWg(),
		"decort_k8s_cp":            k8s.ResourceK8sCP(),
		"decort_snapshot":          snapshot.ResourceSnapshot(),
		"decort_account":           account.ResourceAccount(),
		"decort_bservice":          bservice.ResourceBasicService(),
		"decort_bservice_group":    bservice.ResourceBasicServiceGroup(),
		"decort_image":             image.ResourceImage(),
		"decort_image_virtual":     image.ResourceImageVirtual(),
		"decort_lb":                lb.ResourceLB(),
		"decort_lb_backend":        lb.ResourceLBBackend(),
		"decort_lb_backend_server": lb.ResourceLBBackendServer(),
		"decort_lb_frontend":       lb.ResourceLBFrontend(),
		"decort_lb_frontend_bind":  lb.ResourceLBFrontendBind(),
		"decort_flipgroup":         flipgroup.ResourceFlipgroup(),

		"decort_cb_account":       cb_account.ResourceAccount(),
		"decort_cb_disk":          cb_disks.ResourceDisk(),
		"decort_cb_image":         cb_image.ResourceImage(),
		"decort_cb_virtual_image": cb_image.ResourceVirtualImage(),
		"decort_cb_cdrom_image":   cb_image.ResourceCDROMImage(),
		"decort_cb_delete_images": cb_image.ResourceDeleteImages(),
		"decort_cb_pcidevice":     cb_pcidevice.ResourcePcidevice(),
		"decort_cb_sep":           cb_sep.ResourceSep(),
		"decort_cb_sep_config":    cb_sep.ResourceSepConfig(),
		"decort_cb_resgroup":      cb_rg.ResourceResgroup(),
		"decort_cb_kvmvm":         cb_kvmvm.ResourceCompute(),
		"decort_cb_vins":          cb_vins.ResourceVins(),
		"decort_cb_pfw":           cb_pfw.ResourcePfw(),
		"decort_cb_k8s":           cb_k8s.ResourceK8s(),
		"decort_cb_k8s_wg":        cb_k8s.ResourceK8sWg(),
		"decort_cb_snapshot":      cb_snapshot.ResourceSnapshot(),
	}
}
